@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtegl.c,v 1.11 2003/05/13 04:27:39 nalin Exp $"
+#ident "$Id: vtegl.c,v 1.13 2003/06/01 22:29:20 nalin Exp $"
 
 #include "../config.h"
 
@@ -395,7 +395,8 @@ _vte_gl_set_text_font(struct _vte_draw *draw,
 	}
 	data->cache = _vte_glyph_cache_new();
 
-	_vte_glyph_cache_set_font_description(NULL, data->cache, fontdesc,
+	_vte_glyph_cache_set_font_description(draw->widget,
+					      NULL, data->cache, fontdesc,
 					      _vte_gl_fcpattern_disable_rgba,
 					      NULL);
 }
@@ -513,6 +514,24 @@ _vte_gl_draw_text(struct _vte_draw *draw,
 	glDrawPixels(columns, rows, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
 }
 
+static gboolean
+_vte_gl_draw_char(struct _vte_draw *draw,
+		  struct _vte_draw_text_request *request,
+		  GdkColor *color, guchar alpha)
+{
+	struct _vte_gl_data *data;
+
+	data = (struct _vte_gl_data*) draw->impl_data;
+
+	if (data->cache != NULL) {
+		if (_vte_glyph_get(data->cache, request->c) != NULL) {
+			_vte_gl_draw_text(draw, request, 1, color, alpha);
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static void
 _vte_gl_rectangle(struct _vte_draw *draw,
 		  GLenum type,
@@ -586,6 +605,7 @@ struct _vte_draw_impl _vte_draw_gl = {
 	_vte_gl_get_text_ascent,
 	_vte_gl_get_using_fontconfig,
 	_vte_gl_draw_text,
+	_vte_gl_draw_char,
 	_vte_gl_draw_rectangle,
 	_vte_gl_fill_rectangle,
 	_vte_gl_set_scroll,
